// SPDX-FileCopyrightText: © 2023 Allegro DVT <github-ip@allegrodvt.com>
// SPDX-License-Identifier: MIT

/**************************************************************************//*!
   \addtogroup FourCC
   @{
   \file
 *****************************************************************************/
#pragma once
#include "lib_common/FourCC.h"

/*************************************************************************//*!
   \brief Returns the FOURCC identifier of the decoded frame buffer generated
   by the decoder according to the chroma mode, the bitdepth of the stream
   and the storage mode of the ip
   \param[in] picFmt source picture format
   \return return the corresponding TFourCC format
*****************************************************************************/
TFourCC AL_GetDecFourCC(AL_TPicFormat const picFmt);

/*************************************************************************//*!
   \brief Returns the Chroma ordering format used by the decoder to store
   the specified ChromaMode
   \param[in] eChromaMode decoded picture chroma mode
   \return Returns the corresponding EChromaOrder
*****************************************************************************/
AL_EChromaOrder AL_ChromaModeToChromaOrder(AL_EChromaMode eChromaMode);

/*************************************************************************//*!
   \brief Returns the TPicFormat of the decoded framebuffer generated by the decoder
   according to the chroma mode, the bitdepth of the stream, the storage and
   compression mode of the ip
   \param[in] eChromaMode decoded picture chroma mode
   \param[in] uBitDepth decoded picture bit depth
   \param[in] eStorageMode decoded picture storage mode
   \param[in] bIsCompressed true if decoded picture is compressed, false otherwise
   \return Returns the corresponding TPicFormat
*****************************************************************************/
AL_TPicFormat AL_GetDecPicFormat(AL_EChromaMode eChromaMode, uint8_t uBitDepth, AL_EFbStorageMode eStorageMode, bool bIsCompressed);

/*@}*/
